@@ -40,13 +40,16 @@ class UnexpectedResponse(CacheBackendError):
     pass
 
 class RiakCache(TileCacheBase):
-    def __init__(self, nodes, protocol, bucket, tile_grid, use_secondary_index=False, timeout=60):
+    def __init__(self, nodes, protocol, bucket, tile_grid, use_secondary_index=False, timeout=60, coverage=None):
+        super(RiakCache, self).__init__(coverage)
+        
         if riak is None:
             raise ImportError("Riak backend requires 'riak' package.")
 
         self.nodes = nodes
         self.protocol = protocol
-        self.lock_cache_id = 'riak-' + hashlib.md5(bucket.encode('utf-8')).hexdigest()
+        md5 = hashlib.new('md5', bucket.encode('utf-8'), usedforsecurity=False)
+        self.lock_cache_id = 'riak-' + md5.hexdigest()
         self.request_timeout = timeout * 1000
         self.bucket_name = bucket
         self.tile_grid = tile_grid
