@@ -776,7 +776,7 @@ class WMSSourceConfiguration(SourceConfiguration):
             lock_dir = self.context.globals.get_path('cache.lock_dir', self.conf)
             lock_timeout = self.context.globals.get_value('http.client_timeout', self.conf)
             url = urlparse.urlparse(self.conf['req']['url'])
-            md5 = hashlib.md5(url.netloc.encode('ascii'))
+            md5 = hashlib.new('md5', url.netloc.encode('ascii'), usedforsecurity=False)
             lock_file = os.path.join(lock_dir, md5.hexdigest() + '.lck')
             lock = lambda: SemLock(lock_file, concurrent_requests, timeout=lock_timeout)
 
@@ -922,7 +922,7 @@ class MapnikSourceConfiguration(SourceConfiguration):
             from mapproxy.util.lock import SemLock
             lock_dir = self.context.globals.get_path('cache.lock_dir', self.conf)
             mapfile = self.conf['mapfile']
-            md5 = hashlib.md5(mapfile.encode('utf-8'))
+            md5 = hashlib.new('md5', mapfile.encode('utf-8'), usedforsecurity=False)
             lock_file = os.path.join(lock_dir, md5.hexdigest() + '.lck')
             lock = lambda: SemLock(lock_file, concurrent_requests)
 
@@ -1052,6 +1052,12 @@ def register_source_configuration(config_name, config_class,
 class CacheConfiguration(ConfigurationBase):
     defaults = {'format': 'image/png'}
 
+    @memoize
+    def coverage(self):
+        if not 'cache' in self.conf or not 'coverage' in self.conf['cache']: return None
+        from mapproxy.config.coverage import load_coverage
+        return load_coverage(self.conf['cache']['coverage'])
+
     @memoize
     def cache_dir(self):
         cache_dir = self.conf.get('cache', {}).get('directory')
@@ -1074,11 +1080,13 @@ class CacheConfiguration(ConfigurationBase):
             lock_dir = os.path.join(self.cache_dir(), 'tile_locks')
         return lock_dir
 
-    def _file_cache(self, grid_conf, file_ext):
+    def _file_cache(self, grid_conf, image_opts):
         from mapproxy.cache.file import FileCache
 
         cache_dir = self.cache_dir()
         directory_layout = self.conf.get('cache', {}).get('directory_layout', 'tc')
+        coverage = self.coverage()
+
         if self.conf.get('cache', {}).get('directory'):
             if self.has_multiple_grids():
                 raise ConfigurationError(
@@ -1101,12 +1109,14 @@ class CacheConfiguration(ConfigurationBase):
 
         return FileCache(
             cache_dir,
-            file_ext=file_ext,
+            file_ext=image_opts.format.ext,
+            image_opts=image_opts,
             directory_layout=directory_layout,
             link_single_color_images=link_single_color_images,
+            coverage=coverage
         )
 
-    def _mbtiles_cache(self, grid_conf, file_ext):
+    def _mbtiles_cache(self, grid_conf, image_opts):
         from mapproxy.cache.mbtiles import MBTilesCache
 
         filename = self.conf['cache'].get('filename')
@@ -1120,20 +1130,23 @@ class CacheConfiguration(ConfigurationBase):
 
         sqlite_timeout = self.context.globals.get_value('cache.sqlite_timeout', self.conf)
         wal = self.context.globals.get_value('cache.sqlite_wal', self.conf)
+        coverage = self.coverage()
 
         return MBTilesCache(
             mbfile_path,
             timeout=sqlite_timeout,
             wal=wal,
+            coverage=coverage
         )
 
-    def _geopackage_cache(self, grid_conf, file_ext):
+    def _geopackage_cache(self, grid_conf, image_opts):
         from mapproxy.cache.geopackage import GeopackageCache, GeopackageLevelCache
 
         filename = self.conf['cache'].get('filename')
         table_name = self.conf['cache'].get('table_name') or \
                      "{}_{}".format(self.conf['name'], grid_conf.tile_grid().name)
         levels = self.conf['cache'].get('levels')
+        coverage = self.coverage()
 
         if not filename:
             filename = self.conf['name'] + '.gpkg'
@@ -1158,18 +1171,19 @@ class CacheConfiguration(ConfigurationBase):
 
         if levels:
             return GeopackageLevelCache(
-                cache_dir, grid_conf.tile_grid(), table_name
+                cache_dir, grid_conf.tile_grid(), table_name, coverage=coverage
             )
         else:
             return GeopackageCache(
-                gpkg_file_path, grid_conf.tile_grid(), table_name
+                gpkg_file_path, grid_conf.tile_grid(), table_name, coverage=coverage
             )
 
-    def _azureblob_cache(self, grid_conf, file_ext):
+    def _azureblob_cache(self, grid_conf, image_opts):
         from mapproxy.cache.azureblob import AzureBlobCache
 
         container_name = self.context.globals.get_value('cache.container_name', self.conf,
                                                         global_key='cache.azureblob.container_name')
+        coverage = self.coverage()
 
         if not container_name:
             raise ConfigurationError("no container_name configured for Azure Blob cache %s" % self.conf['name'])
@@ -1188,17 +1202,19 @@ class CacheConfiguration(ConfigurationBase):
 
         return AzureBlobCache(
             base_path=base_path,
-            file_ext=file_ext,
+            file_ext=image_opts.format.ext,
             directory_layout=directory_layout,
             container_name=container_name,
-            connection_string=connection_string
+            connection_string=connection_string,
+            coverage=coverage
         )
 
-    def _s3_cache(self, grid_conf, file_ext):
+    def _s3_cache(self, grid_conf, image_opts):
         from mapproxy.cache.s3 import S3Cache
 
         bucket_name = self.context.globals.get_value('cache.bucket_name', self.conf,
             global_key='cache.s3.bucket_name')
+        coverage = self.coverage()
 
         if not bucket_name:
             raise ConfigurationError("no bucket_name configured for s3 cache %s" % self.conf['name'])
@@ -1215,6 +1231,10 @@ class CacheConfiguration(ConfigurationBase):
         access_control_list = self.context.globals.get_value('cache.access_control_list', self.conf,
             global_key='cache.s3.access_control_list')
 
+        use_http_get = self.context.globals.get_value('cache.use_http_get', self.conf,
+            global_key='cache.s3.use_http_get'
+        )
+
         directory_layout = self.conf['cache'].get('directory_layout', 'tms')
 
         base_path = self.conf['cache'].get('directory', None)
@@ -1223,16 +1243,18 @@ class CacheConfiguration(ConfigurationBase):
 
         return S3Cache(
             base_path=base_path,
-            file_ext=file_ext,
+            file_ext=image_opts.format.ext,
             directory_layout=directory_layout,
             bucket_name=bucket_name,
             profile_name=profile_name,
             region_name=region_name,
             endpoint_url=endpoint_url,
-            access_control_list=access_control_list
+            access_control_list=access_control_list,
+            coverage=coverage,
+            use_http_get=use_http_get
         )
 
-    def _sqlite_cache(self, grid_conf, file_ext):
+    def _sqlite_cache(self, grid_conf, image_opts):
         from mapproxy.cache.mbtiles import MBTilesLevelCache
 
         cache_dir = self.conf.get('cache', {}).get('directory')
@@ -1251,14 +1273,17 @@ class CacheConfiguration(ConfigurationBase):
 
         sqlite_timeout = self.context.globals.get_value('cache.sqlite_timeout', self.conf)
         wal = self.context.globals.get_value('cache.sqlite_wal', self.conf)
+        coverage = self.coverage()
 
         return MBTilesLevelCache(
             cache_dir,
             timeout=sqlite_timeout,
             wal=wal,
+            ttl=self.conf.get('cache', {}).get('ttl', 0),
+            coverage=coverage
         )
 
-    def _couchdb_cache(self, grid_conf, file_ext):
+    def _couchdb_cache(self, grid_conf, image_opts):
         from mapproxy.cache.couchdb import CouchDBCache, CouchDBMDTemplate
 
         db_name = self.conf['cache'].get('db_name')
@@ -1272,17 +1297,19 @@ class CacheConfiguration(ConfigurationBase):
 
         md_template = CouchDBMDTemplate(self.conf['cache'].get('tile_metadata', {}))
         tile_id = self.conf['cache'].get('tile_id')
+        coverage = self.coverage()
 
         return CouchDBCache(url=url, db_name=db_name,
-            file_ext=file_ext, tile_grid=grid_conf.tile_grid(),
-            md_template=md_template, tile_id_template=tile_id)
+            file_ext=image_opts.format.ext, tile_grid=grid_conf.tile_grid(),
+            md_template=md_template, tile_id_template=tile_id, coverage=coverage)
 
-    def _riak_cache(self, grid_conf, file_ext):
+    def _riak_cache(self, grid_conf, image_opts):
         from mapproxy.cache.riak import RiakCache
 
         default_ports = self.conf['cache'].get('default_ports', {})
         default_pb_port = default_ports.get('pb', 8087)
         default_http_port = default_ports.get('http', 8098)
+        coverage = self.coverage()
 
         nodes = self.conf['cache'].get('nodes')
         if not nodes:
@@ -1306,17 +1333,23 @@ class CacheConfiguration(ConfigurationBase):
         return RiakCache(nodes=nodes, protocol=protocol, bucket=bucket,
             tile_grid=grid_conf.tile_grid(),
             use_secondary_index=use_secondary_index,
-            timeout=timeout
+            timeout=timeout,
+            coverage=coverage
         )
 
-    def _redis_cache(self, grid_conf, file_ext):
+    def _redis_cache(self, grid_conf, image_opts):
         from mapproxy.cache.redis import RedisCache
 
         host = self.conf['cache'].get('host', '127.0.0.1')
         port = self.conf['cache'].get('port', 6379)
         db = self.conf['cache'].get('db', 0)
         ttl = self.conf['cache'].get('default_ttl', 3600)
-
+        username = self.conf['cache'].get('username',None)
+        password = self.conf['cache'].get('password', None)
+        coverage = self.coverage()
+        ssl_certfile = self.conf['cache'].get('ssl_certfile',None)
+        ssl_keyfile = self.conf['cache'].get('ssl_keyfile', None)
+        ssl_ca_certs = self.conf['cache'].get('ssl_ca_certs', None)
         prefix = self.conf['cache'].get('prefix')
         if not prefix:
             prefix = self.conf['name'] + '_' + grid_conf.tile_grid().name
@@ -1325,13 +1358,20 @@ class CacheConfiguration(ConfigurationBase):
             host=host,
             port=port,
             db=db,
+            username=username,
+            password=password,
             prefix=prefix,
             ttl=ttl,
+            coverage=coverage,
+            ssl_certfile=ssl_certfile,
+            ssl_keyfile=ssl_keyfile,
+            ssl_ca_certs=ssl_ca_certs
         )
 
-    def _compact_cache(self, grid_conf, file_ext):
+    def _compact_cache(self, grid_conf, image_opts):
         from mapproxy.cache.compact import CompactCacheV1, CompactCacheV2
 
+        coverage = self.coverage()
         cache_dir = self.cache_dir()
         if self.conf.get('cache', {}).get('directory'):
             if self.has_multiple_grids():
@@ -1345,20 +1385,20 @@ class CacheConfiguration(ConfigurationBase):
 
         version = self.conf['cache']['version']
         if version == 1:
-            return CompactCacheV1(cache_dir=cache_dir)
+            return CompactCacheV1(cache_dir=cache_dir, coverage=coverage)
         elif version == 2:
-            return CompactCacheV2(cache_dir=cache_dir)
+            return CompactCacheV2(cache_dir=cache_dir, coverage=coverage)
 
         raise ConfigurationError("compact cache only supports version 1 or 2")
 
-    def _tile_cache(self, grid_conf, file_ext):
+    def _tile_cache(self, grid_conf, image_opts):
         if self.conf.get('disable_storage', False):
             from mapproxy.cache.dummy import DummyCache
             return DummyCache()
 
         grid_conf.tile_grid() #create to resolve `base` in grid_conf.conf
         cache_type = self.conf.get('cache', {}).get('type', 'file')
-        return getattr(self, '_%s_cache' % cache_type)(grid_conf, file_ext)
+        return getattr(self, '_%s_cache' % cache_type)(grid_conf, image_opts)
 
     def _tile_filter(self):
         filters = []
@@ -1577,7 +1617,7 @@ class CacheConfiguration(ConfigurationBase):
             tile_grid = grid_conf.tile_grid()
             tile_filter = self._tile_filter()
             image_opts = compatible_image_options(source_image_opts, base_opts=base_image_opts)
-            cache = self._tile_cache(grid_conf, image_opts.format.ext)
+            cache = self._tile_cache(grid_conf, image_opts)
             identifier = self.conf['name'] + '_' + tile_grid.name
 
             tile_creator_class = None
@@ -1640,7 +1680,10 @@ class CacheConfiguration(ConfigurationBase):
             if self.conf['name'] in self.context.caches:
                 mgr._refresh_before = self.context.caches[self.conf['name']].conf.get('refresh_before', {})
             extent = merge_layer_extents(sources)
-            if extent.is_default:
+            # If the cache has a defined coverage prefer it's extent over source extent
+            if cache.coverage:
+                extent = cache.coverage.extent
+            elif extent.is_default:
                 extent = map_extent_from_grid(tile_grid)
             caches.append((tile_grid, extent, mgr))
         return caches
@@ -2164,8 +2207,11 @@ class ServiceConfiguration(ConfigurationBase):
                 # demo service only supports 1.1.1, use wms_111 as an indicator
                 services.append('wms_111')
 
+        layers = odict(sorted(layers.items(), key=lambda x: x[1].name))
+        background =  self.context.globals.get_value('background', conf)
+
         return DemoServer(layers, md, tile_layers=tile_layers,
-            image_formats=image_formats, srs=srs, services=services, restful_template=restful_template)
+            image_formats=image_formats, srs=srs, services=services, restful_template=restful_template, background = background)
 
 
 def load_plugins():
