@@ -14,17 +14,23 @@
 # limitations under the License.
 
 import copy
+import json
+
+from functools import reduce
 from io import StringIO
-from mapproxy.compat import string_type, PY2, BytesIO
+
+from mapproxy.compat import string_type, PY2, BytesIO, iteritems
 
 try:
     from lxml import etree, html
+
     has_xslt_support = True
-    etree, html # prevent pyflakes warning
+    etree, html  # prevent pyflakes warning
 except ImportError:
     has_xslt_support = False
     etree = html = None
 
+
 class FeatureInfoDoc(object):
     content_type = None
 
@@ -36,7 +42,7 @@ class FeatureInfoDoc(object):
 
 
 class TextFeatureInfoDoc(FeatureInfoDoc):
-    info_type = 'text'
+    info_type = "text"
 
     def __init__(self, content):
         self.content = content
@@ -47,10 +53,12 @@ class TextFeatureInfoDoc(FeatureInfoDoc):
     @classmethod
     def combine(cls, docs):
         result_content = [doc.as_string() for doc in docs]
-        return cls(b'\n'.join(result_content))
+        return cls(b"\n".join(result_content))
+
 
 class XMLFeatureInfoDoc(FeatureInfoDoc):
-    info_type = 'xml'
+    info_type = "xml"
+    defaultEncoding = "UTF-8"
 
     def __init__(self, content):
         if isinstance(content, (string_type, bytes)):
@@ -58,10 +66,10 @@ class XMLFeatureInfoDoc(FeatureInfoDoc):
             self._etree = None
         else:
             self._str_content = None
-            if hasattr(content, 'getroottree'):
+            if hasattr(content, "getroottree"):
                 content = content.getroottree()
             self._etree = content
-            assert hasattr(content, 'getroot'), "expected etree like object"
+            assert hasattr(content, "getroot"), "expected etree like object"
 
     def as_string(self):
         if self._str_content is None:
@@ -74,7 +82,9 @@ class XMLFeatureInfoDoc(FeatureInfoDoc):
         return self._etree
 
     def _serialize_etree(self):
-        return etree.tostring(self._etree)
+        encoding = self._etree.docinfo.encoding if \
+            self._etree.docinfo.encoding else self.defaultEncoding
+        return etree.tostring(self._etree, encoding=encoding, xml_declaration=False)
 
     def _parse_content(self):
         doc = as_io(self._str_content)
@@ -82,7 +92,8 @@ class XMLFeatureInfoDoc(FeatureInfoDoc):
 
     @classmethod
     def combine(cls, docs):
-        if etree is None: return TextFeatureInfoDoc.combine(docs)
+        if etree is None:
+            return TextFeatureInfoDoc.combine(docs)
         doc = docs.pop(0)
         result_tree = copy.deepcopy(doc.as_etree())
         for doc in docs:
@@ -91,8 +102,9 @@ class XMLFeatureInfoDoc(FeatureInfoDoc):
 
         return cls(result_tree)
 
+
 class HTMLFeatureInfoDoc(XMLFeatureInfoDoc):
-    info_type = 'html'
+    info_type = "html"
 
     def _parse_content(self):
         root = html.document_fromstring(self._str_content)
@@ -120,29 +132,90 @@ class HTMLFeatureInfoDoc(XMLFeatureInfoDoc):
 
         return cls(result_tree)
 
+
+class JSONFeatureInfoDoc(FeatureInfoDoc):
+    info_type = "json"
+
+    def __init__(self, content):
+        self.content = content
+
+    def as_string(self):
+        return self.content
+
+    @classmethod
+    def combine(cls, docs):
+        contents = []
+        for d in docs:
+            content = d.content
+            if not isinstance(content, string_type):
+                content = content.decode('UTF-8')
+            contents.append(json.loads(content))
+        combined = reduce(lambda a, b: merge_dict(a, b), contents)
+        return cls(json.dumps(combined))
+
+
+def merge_dict(base, other):
+    """
+    Return `base` dict with values from `conf` merged in.
+    """
+    for k, v in iteritems(other):
+        if k not in base:
+            base[k] = v
+        else:
+            if isinstance(base[k], dict):
+                merge_dict(base[k], v)
+            elif isinstance(base[k], list):
+                base[k].extend(v)
+            else:
+                base[k] = v
+    return base
+
+
 def create_featureinfo_doc(content, info_format):
-    info_format = info_format.split(';', 1)[0].strip() # remove mime options like charset
-    if info_format in ('text/xml', 'application/vnd.ogc.gml'):
+    info_type = featureinfo_type(info_format)
+    if info_type == "xml":
         return XMLFeatureInfoDoc(content)
-    if info_format == 'text/html':
+    if info_type == "html":
         return HTMLFeatureInfoDoc(content)
+    if info_type == "json":
+        return JSONFeatureInfoDoc(content)
 
     return TextFeatureInfoDoc(content)
 
+def featureinfo_type(info_format):
+    info_format = info_format.split(";", 1)[
+        0
+    ].strip()  # remove mime options like charset
+    if info_format in ("text/xml", "application/xml",
+                       "application/gml+xml", "application/vnd.ogc.gml"):
+        return "xml"
+    if info_format == "text/html":
+        return "html"
+    if info_format == "application/json":
+        return "json"
+
+    return "text"
+
 
 class XSLTransformer(object):
-    def __init__(self, xsltscript):
+
+    def __init__(self, xsltscript, info_format=None):
         self.xsltscript = xsltscript
+        self.info_type = featureinfo_type(info_format or "text/xml")
 
     def transform(self, input_doc):
         input_tree = input_doc.as_etree()
         xslt_tree = etree.parse(self.xsltscript)
         transform = etree.XSLT(xslt_tree)
         output_tree = transform(input_tree)
-        return XMLFeatureInfoDoc(output_tree)
+        if self.info_type == "html":
+            return HTMLFeatureInfoDoc(output_tree)
+        else:
+            return XMLFeatureInfoDoc(output_tree)
 
     __call__ = transform
 
+
 def as_io(doc):
     if PY2:
         return BytesIO(doc)
@@ -153,10 +226,27 @@ def as_io(doc):
             return BytesIO(doc)
 
 
-def combined_inputs(input_docs):
-    doc = input_docs.pop(0)
-    input_tree = etree.parse(as_io(doc))
-    for doc in input_docs:
-        doc_tree = etree.parse(as_io(doc))
-        input_tree.getroot().extend(doc_tree.getroot().iterchildren())
-    return input_tree
+def combine_docs(docs, transformer=None):
+    """
+    Combine multiple FeatureInfoDocs.
+
+    Combines as text, if the type of the docs differ.
+    Otherwise the type specifix combine is called.
+
+    Returns the combined document and the info_type (text, xml or json).
+    info_type is None if the output is transformed, as the type is dependent on the
+    transformer.
+    """
+    if len(set(d.info_type for d in docs)) > 1:
+        # more then one info_type, combine as plain text
+        doc = TextFeatureInfoDoc.combine(docs)
+        infotype = "text"
+    else:
+        # all same type, combine with type specific handler
+        infotype = docs[0].info_type
+        doc = docs[0].combine(docs)
+
+        if transformer:
+            doc = transformer(doc)
+            infotype = None # defined by transformer
+    return doc.as_string(), infotype
