@@ -45,7 +45,6 @@ from mapproxy.layer import DefaultMapExtent, MapExtent
 
 get_template = template_loader(__name__, 'templates', namespace=template_helper.__dict__)
 
-
 class PERMIT_ALL_LAYERS(object):
     pass
 
@@ -83,7 +82,7 @@ class WMSServer(Server):
         self.check_map_request(map_request)
 
         params = map_request.params
-        query = MapQuery(params.bbox, params.size, SRS(params.srs), params.format)
+        query = MapQuery(params.bbox, params.size, SRS(params.srs), params.format, dimensions=map_request.dimensions)
 
         if map_request.params.get('tiled', 'false').lower() == 'true':
             query.tiled_only = True
@@ -307,11 +306,16 @@ class WMSServer(Server):
         legend = self.layers[layer].legend(request)
 
         [legends.append(i) for i in legend if i is not None]
-        result = concat_legends(legends)
         if 'format' in request.params:
             mimetype = request.params.format_mime_type
         else:
             mimetype = 'image/png'
+
+        if mimetype == 'application/json':
+            return Response(legends[0].encode(), mimetype='application/json')
+
+        result = concat_legends(legends)
+
         img_opts = self.image_formats[request.params.format_mime_type]
         return Response(result.as_buffer(img_opts), mimetype=mimetype)
 
@@ -511,8 +515,10 @@ class Capabilities(object):
 
     def layer_llbbox(self, layer):
         if 'EPSG:4326' in self.srs_extents:
-            llbbox = self.srs_extents['EPSG:4326'].intersection(layer.extent).llbbox
-            return limit_llbbox(llbbox)
+            intersection = self.srs_extents['EPSG:4326'].intersection(layer.extent)
+            if intersection is not None:
+                llbbox = intersection.llbbox
+                return limit_llbbox(llbbox)
         return limit_llbbox(layer.extent.llbbox)
 
     def render(self, _map_request):
@@ -695,7 +701,7 @@ class WMSLayer(WMSLayerBase):
     is_active = True
     layers = []
     def __init__(self, name, title, map_layers, info_layers=[], legend_layers=[],
-                 res_range=None, md=None):
+                 res_range=None, md=None,dimensions=None):
         self.name = name
         self.title = title
         self.md = md or {}
@@ -703,11 +709,14 @@ class WMSLayer(WMSLayerBase):
         self.info_layers = info_layers
         self.legend_layers = legend_layers
         self.extent = merge_layer_extents(map_layers)
+        self.dimensions = dimensions
+
         if res_range is None:
             res_range = merge_layer_res_ranges(map_layers)
         self.res_range = res_range
         self.queryable = True if info_layers else False
         self.has_legend = True if legend_layers else False
+        self.dimensions = dimensions
 
     def is_opaque(self, query):
         return any(l.is_opaque(query) for l in self.map_layers)
